%% Matlab code to calculate the SEA example in Chapter 12
% Carl Howard 18/3/2017
%
% This script is used to calculate the SEA example in the ENC 5th edition.

%%    STEP 1: Parameters
% ==============================

% Material properties for steel

steel_E = 207e9;    % [Pa] Youngs modulus
steel_nu = 0.3;     % [] Poisson's ratio
steel_rho = 7850;   % [kg/m^3]
steel_c = sqrt(steel_E/steel_rho);     % [m/s] speed of sound in steel

% Material properties for air
air_rho = 1.21;     % [kg/m^3]  density of air
air_c = 343;        % [m/s] speed of sound in air

% Geometry of the beam
beam_L = 2;             %[m] length
beam_width = 20e-3;     %[m] width
beam_height = 20e-3;    %[m] height
beam_area = beam_width*beam_height;     % [m^2] cross sectional area of beam
% moments of inertia
beam_Ixx= beam_width*beam_height^3/12;   %[m^4]
beam_Iyy= beam_width^3*beam_height/12;   %[m^4]
beam_eta = 0.15;         % [] damping loss factor for the beam

% Geometry of the plate
plate_Lx = 3;                       %[m] length along x
plate_Ly = 3;                       %[m] length along y
plate_thick = 20e-3;                %[m] plate thickness
plate_eta = 0.15;                   % [] damping loss factor for the plate
plate_D = steel_E*plate_thick^3/(12*(1-steel_nu^2));    % [] Bending stiffness of the plate
plate_A = plate_Lx * plate_Ly;      %[m^2] area of plate
plate_P = 2*(plate_Lx+plate_Ly);    %[m] Perimeter of the plate
plate_cL= sqrt(steel_E/(steel_rho*(1-steel_nu^2)));



% Room Geometry
room_V = 5^3;                       % [m^3] Volume of the room
room_A = 6*5^2;                     % [m^2] Area of the room
room_P = 4*(3*5);                   % [m] Perimeter of the room
room_alpha = 0.07;                  % [] absorption of the room


% Frequencies of analysis [Hz]
f=[63 ; 125 ;250 ; 500 ; 1000 ; 2000 ];

% ==============================
%%    STEP 2: System Diagram 
% ==============================
% See book

% ==============================
%%    STEP 3: Impedances
% ==============================
c_b_beam = (steel_E * beam_Ixx/(steel_rho*beam_area) * (2*pi*f).^2 ).^(0.25);
Z_beam = (1-j)*steel_rho*beam_area/2* (c_b_beam.^3 ./ (2*pi*f).^2);

% Z_plate = 2.3 * steel_c* steel_rho * plate_thick^2;
% Z_plate=ones(size(f))*Z_plate;

% Bending wave speed in the plate
c_b_plate = sqrt(2*pi*f) * (plate_D/(steel_rho*plate_thick))^(0.25);  %[m/s]


% Z_plate = 16*steel_rho*plate_thick*c_b_plate.^4 ./ (2*pi*f).^3 .*...
%             (   ones(size(f)) ./   ...
%                 (ones(size(f)) - 4/pi*log(0.9*2*pi*f*beam_height./c_b_plate))   ...
%             );

% Note the calculation of the impedance of a plate involves the 
% calculation of a "natural" logarithm, base e.
% The Matlab command for calculating log_e( ), which is sometimes 
% written in books as ln, is log(  ).
% The base 10 logarithm, log_10, which is commonly used in acoustics and 
% vibrations, is the Matlab command log10(  ).
Z_plate = 16*plate_D./(2*pi*f) .*...
            (   ones(size(f)) ./   ...
                (ones(size(f)) - 4*j/pi*log(0.89*beam_height*(2*pi*f)./c_b_plate))   ...
            );
        
        
%%    STEP 4: Modal Densities
% ==============================

% Modal Density of the beam
n_1= beam_L / (2*pi) * (steel_rho*beam_area / (steel_E*beam_Ixx) )^(0.25) * ...
        sqrt(ones(size(f)) ./ (2*pi*f) );
% n_1 checked
    
% Modal Density of the plate
n_2=sqrt(3) * plate_A / (2*pi*plate_thick*plate_cL) * ones(size(f));
% n_2 close. acceptable. example uses different formula 
%n_2_2=plate_Lx*plate_Ly/(4*pi)*sqrt(steel_rho*plate_thick / plate_D) * ones(size(f)); 

% Modal Density of the room
n_3=    room_V/(2*pi^2*air_c^3)*(2*pi*f).^2 +    ...
        room_A/(8*pi*air_c^2)*(2*pi*f)    +       ...
        room_P/(16*pi*air_c);


% ==============================
%%    STEP 5: Coupling Loss Factors
% ==============================

% beam to plate
% ------------------------------

% This equation came from Anthony Zander's assignment
%eta_12 = 4*steel_rho/(beam_area*steel_rho*2*pi) *(plate_cb./f);

% eta_12 = 2*ones(size(f)) ./ (pi*(2*pi*f).*n_1) .*     ...
%             real(Z_beam).*real(Z_plate) ./ abs(Z_beam+Z_plate).^2 ;

% FreeSEA seems to think this should be 4*, rather than 2*.
% This might be right as Lyon p191 has a transmission coefficient with 4*
% out the front... but this is not CLF which is eta_12... need to keep
% looking for other references.
% Lyon p74 3.3.17 looks like he uses 2*.
eta_12 = 4*ones(size(f)) ./ (pi*(2*pi*f).*n_1) .*     ...
            real(Z_beam).*real(Z_plate) ./ abs(Z_beam+Z_plate).^2 ;


    
    
% plate to room
% ------------------------------
% Calculate the critical frequency in the plate
% Not used in the calculations directly, but is used to look up the graphs
% for the radiation efficiency
plate_f_critical = (air_c^2 / (2*pi)) * sqrt(12*steel_rho*(1-steel_nu^2)/(steel_E*plate_thick^2));
plate_f_critical2 = 0.551*air_c^2/(steel_c*plate_thick);

% Using Eq 7.6 in the 5th edition, the longitudinal wave speed is
%
% $$ c_L = \frac{\sqrt{12}}{h} \sqrt{ \frac{B}{m} } $$
%
plate_cL2 = sqrt(12)/plate_thick * sqrt( plate_D/ (steel_rho*plate_thick));

% Radiation efficiency from chart in Bies and Hansen
sigma_rad_db = [-15, -14 , -10, -1, 2 , 0.0 ];

sigma_rad = 10.^(sigma_rad_db / 10);
eta_23 = air_rho*air_c*plate_A/(steel_rho*plate_A*plate_thick)  ...
            * sigma_rad.' ./ (2*pi*f);

% Calculate the reciprocal CLFs
eta_21 = eta_12 .* n_1 ./ n_2;
eta_32 = eta_23 .* n_2 ./ n_3;



% ==============================
%%    STEP 6: Damping Loss Factors
% ==============================
% Damping Loss factors
eta_1=beam_eta*ones(size(f));       % DLF for the beam
eta_2=plate_eta*ones(size(f));       % DLF for the plate
eta_3=air_c * room_A * room_alpha / (8 * pi * room_V) * ones(size(f))./f;





% ==============================
%%    STEP 7: Input Powers
% ==============================
% Power into the sub-systems.
% P_1 is for the beam.
P_1 = 1;
P_2 = 0;
P_3 = 0;

Power = [ P_1 ; P_2 ; P_3];

for ii=1:length(f),

% ==============================
%%    STEP 8: Matrix Equation
% ==============================
    
    % Create the matrix equation
    LHS = [ (eta_1(ii) + eta_12(ii))*n_1(ii)    ,       -(eta_12(ii) * n_1(ii))             ,   0       ;   ...
            (-eta_21(ii)*n_2(ii))           ,   (eta_2(ii) + eta_21(ii) + eta_23(ii))*n_2(ii)   ,   (-eta_23(ii)*n_2(ii))       ;   ...
            0                       ,   (-eta_32(ii)*n_3(ii))                   ,   (eta_3(ii) + eta_32(ii))*n_3(ii)    ];

    RHS = Power;
    Energy(:,ii) = inv(2*pi*f(ii)*LHS)*Power;
end;



% ==============================
%%    STEP 9: Amplitude Responses
% ==============================


% Calculate the energies in the sub-systems
E=Energy.*[n_1.' ; n_2.'; n_3.'];

% The average sound pressure level in the room is given by
% p^2 = rho c_0^2 * E / V
p_3_sq = air_rho * air_c^2 * (E(3,:)) / room_V;
SPL=10*log10(p_3_sq/(20e-6)^2);

v_sq_beam = E(1,:) / (beam_L*beam_width*beam_height*steel_rho);
v_sq_plate = E(2,:) / (plate_thick*plate_Lx*plate_Ly*steel_rho);

v_beam = sqrt(v_sq_beam);
v_plate = sqrt(v_sq_plate);

% For reasons that I don't understand, FreeSEA uses a reference value
% for structural systems of 5e-8 when calculating the dB values.
% Uncomment these lines if you want to compare the matlab results 
% in dB re 5e-8m
% dB_v_sq_beam = 10*log10(abs(v_sq_beam) ./ (5e-8)^2 );
% dB_v_sq_plate = 10*log10(abs(v_sq_plate) ./ (5e-8)^2 );
% Otherwise if you want to see the results in dB re 1e-9m
% then uncomment these lines
dB_v_sq_beam = 10*log10(abs(v_sq_beam) ./ (1e-9)^2 );
dB_v_sq_plate = 10*log10(abs(v_sq_plate) ./ (1e-9)^2 );


% Display results
disp(' ');
disp('Calculated sound pressure levels and velocities for SEA Example');
disp('[    f     SPL  dB_v_sq_beam  dB_v_sq_plate ]');
tablexy=[f.' ;   SPL ;  dB_v_sq_beam ; dB_v_sq_plate ];
fprintf('%6.0f   %6.2f   %6.2f      %6.2f\n',tablexy);


disp(' ');
disp('Radiation efficiencies and loss factors for SEA Example');
disp('[f  sigma_rad_db   eta_12   eta_23 ]');
tablexy=[f.' ; sigma_rad_db ;   eta_12.'  ;   eta_23.' ];
fprintf('%6.0f   %6.1f  %6.4e   %6.4e \n',tablexy);
